#<profile4xi profile = "02" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "boundedsemaphore4xi.py">
#<abstract4xi abstract = "provides BoundedSemaphore4Xi class"/>
#<refer4xi reference = "synchronize.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------
#

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'BoundedSemaphore4Xi' ]
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:
    import _winapi
except:
    pass    
try:                 # imported-as ones 
    from time import time as _time, sleep as _sleep         
except:
    pass            
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    pass
try:                             # Python ones
    from multiprocessing.synchronize import BoundedSemaphore
except:
    print("boundedsemaphore4xi.py: *** failed in multiprocessing.synchronize import BoundedSemaphore ***")
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from constant_synchronize4xi import ConstantSynchronize4Xi 
from constant_synchronize4xi import get_semaphore_synchronize4xi    
from constant_synchronize4xi import get_sem_value_synchronize4xi    
from constant_synchronize4xi import get_sem_value_max_synchronize4xi   
from examine4xi import examine4xi
from print4xi import print4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi
from semaphore4xi import Semaphore4Xi
from semlock4xi import SemLock4Xi
from time4xi import time4xi
#"
#/>        

# =================================================================================================
#<class4xi class = "BoundedSemaphore4Xi" base = "Semaphore4Xi" child = "None"
#  synopsis = "Provides bounded semaphore services"
#  usage = "
#
#   queue4xi.py:
#       self._sem = BoundedSemaphore4Xi(maxsize)
#
#   syncmanager4xi.py:
#     SyncManager4Xi.register('BoundedSemaphore4Xi', threading.BoundedSemaphore, AcquirerProxy4Xi)
#"
#> 
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# =================================================================================================
#
class BoundedSemaphore4Xi(Semaphore4Xi): 
    '''
    Provides bounded semaphore services
    '''

    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi exposure = "
    _exposed_ = ('sense', 'reason', 'ace')          
    #"
    #/>
                                  
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-BoundedSemaphore4Xi" usage = "initializes BoundedSemaphore4Xi">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __init__(self, value=1): 
        '''
        BoundedSemaphore4Xi method __init__: initializes with (self)
        '''
        class2print = 'BoundedSemaphore4Xi'   
        method2print = '__init__'   
                
        # -------------------------------------------------------------------------------------------------------------------------------------------
        # message2print = "{(Step 1/7a) stepping" + "; (Step 1/7)-stepping-by-init-of-BoundedSemaphore4Xi: " + "~~"  + "; time: " + time4xi() + "}"
        # print4xi(class2print, method2print, message2print)
        # -------------------------------------------------------------------------------------------------------------------------------------------
                    
        class2print = 'BoundedSemaphore4Xi'   
        method2print = '__init__'        
        
        message2print = "{(Step 1/7b) started" + "; init-of-BoundedSemaphore4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              

        message2print = "{(Step 1/7c) passed; value-passed-to-init-of-BoundedSemaphore4Xi: " + str(value) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        
        message2print = "{(Step 1/7d) borning" + "; boundedsemaphore2xi-of-BoundedSemaphore4Xi-borning: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        message2print = "{(Step 1/7e) inheriting" + "; Semaphore4Xi-inheriting-by-boundedsemaphore2xi-of-BoundedSemaphore4Xi: " + str(Semaphore4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        
        if examine4xi(class2print, method2print):
        
            # -------------------------------------------------------------------------------------    
            # OK StepOriginal
            # -------------------------------------------------------------------------------------             
            message2print = "{(StepOriginal) calling" + "; BoundedSemaphore-init-calling-by-init-of-BoundedSemaphore4Xi: " + str(BoundedSemaphore.__init__) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       

            BoundedSemaphore.__init__(self, value=1)
            
        else:                
            # -------------------------------------------------------------------------------------------------------------------------------------------
            # message2print = "{(Step 2/7a) stepping" + "; (Step 2/7)-stepping-by-init-of-BoundedSemaphore4Xi: " + "~~"  + "; time: " + time4xi() + "}"
            # print4xi(class2print, method2print, message2print)
            # -------------------------------------------------------------------------------------------------------------------------------------------
               
            self.csyn = ConstantSynchronize4Xi()
            
            message2print = "{(Step 2/7b) created" + "; csyn2xi-of-boundedsemaphore2xi-created-by-init-of-BoundedSemaphore4Xi: " + str(self.csyn) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
                    
            # -------------------------------------------------------------------------------------------------------------------------------------------
            # message2print = "{(Step 3/7a) stepping" + "; (Step 3/7)-stepping-by-init-of-BoundedSemaphore4Xi: " + "~~"  + "; time: " + time4xi() + "}"
            # print4xi(class2print, method2print, message2print)
            # -------------------------------------------------------------------------------------------------------------------------------------------
                    
            self.semaphore = self.csyn.get_semaphore()
    
            message2print = "{(Step 3/7b) set; semaphore-of-boundedsemaphore2xi-set-by-init-of-BoundedSemaphore4Xi to: " + str(self.semaphore) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
                    
            # -------------------------------------------------------------------------------------------------------------------------------------------
            # message2print = "{(Step 4/7a) stepping" + "; (Step 4/7)-stepping-by-init-of-BoundedSemaphore4Xi: " + "~~"  + "; time: " + time4xi() + "}"
            # print4xi(class2print, method2print, message2print)
            # -------------------------------------------------------------------------------------------------------------------------------------------
                            
            self.maxvalue = get_sem_value_max_synchronize4xi()
    
            message2print = "{(Step 4/7b) set; maxvalue-of-boundedsemaphore2xi-set-by-init-of-BoundedSemaphore4Xi to: " + str(self.maxvalue) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
                    
            # -------------------------------------------------------------------------------------------------------------------------------------------
            # message2print = "{(Step 5/7a) stepping" + "; (Step 5/7)-stepping-by-init-of-BoundedSemaphore4Xi: " + "~~"  + "; time: " + time4xi() + "}"
            # print4xi(class2print, method2print, message2print)
            # -------------------------------------------------------------------------------------------------------------------------------------------
                            
            self._semlock = _multiprocessing.SemLock(get_semaphore_synchronize4xi(), value, self.maxvalue)
    
            
            message2print = "{(Step 5/7b) set; semlock-of-boundedsemaphore2xi-set-by-init-of-BoundedSemaphore4Xi to: " + str(self._semlock) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
         
                    
            # -------------------------------------------------------------------------------------------------------------------------------------------
            # message2print = "{(Step 6/7a) stepping" + "; (Step 6/7)-stepping-by-init-of-BoundedSemaphore4Xi: " + "~~"  + "; time: " + time4xi() + "}"
            # print4xi(class2print, method2print, message2print)
            # -------------------------------------------------------------------------------------------------------------------------------------------
            message2print = "{(Step 6/7b) passing" + "; semaphore-of-boundedsemaphore2xi-passing-to-SemLock4Xi-init-by-init-of-BoundedSemaphore4Xi: " + str(self.semaphore) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                    
            message2print = "{(Step 6/7c) passing" + "; value-passing-to-SemLock4Xi-init-by-init-of-BoundedSemaphore4Xi: " + str(value) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                               
    
            message2print = "{(Step 6/7d) passing" + "; maxvalue-of-boundedsemaphore2xi-passing-to-SemLock4Xi-init-by-init-of-BoundedSemaphore4Xi: " + str(self.maxvalue) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            
            SemLock4Xi.__init__(self, self.semaphore, value, self.maxvalue)      
                                                                                
            message2print = "{performed" + "; SemLock4Xi-init-performed-by-init-of-BoundedSemaphore4Xi: " + str(SemLock4Xi.__init__) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
                    
            # -------------------------------------------------------------------------------------------------------------------------------------------
            # message2print = "{(Step 7/7a) stepping" + "; (Step 7/7)-stepping-by-init-of-BoundedSemaphore4Xi: " + "~~"  + "; time: " + time4xi() + "}"
            # print4xi(class2print, method2print, message2print)
            # -------------------------------------------------------------------------------------------------------------------------------------------
            
            
            message2print = "{(Step 7/7b) inherited" + "; Semaphore4Xi-inherited-by-boundedsemaphore2xi-of-BoundedSemaphore4Xi: " + str(Semaphore4Xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
                        
            message2print = "{(Step 7/7c) activated" + "; boundedsemaphore2xi-of-BoundedSemaphore4Xi-activated: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            
            message2print = "{(Step 7/7d) returning" + "; init-of-BoundedSemaphore4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
            
            # -----------------------------------------------------------------------------------------    
            #
            # ----------------------------------------------------------------------------------------- 
        
        return
                
    #</methodize4xi>
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "repr-of-BoundedSemaphore4Xi" usage = "proxies with (self)">                         
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __repr__(self):
        '''
        BoundedSemaphore4Xi method __repr__: proxies with (self)  
        '''
        class2print = 'BoundedSemaphore4Xi'   
        method2print = '__repr__'         
        
                
        # -------------------------------------------------------------------------------------------------------------------------------------------
        # message2print = "{(Step 1/3a) stepping" + "; (Step 1/3)-stepping-by-repr-of-BoundedSemaphore4Xi: " + "~~"  + "; time: " + time4xi() + "}"
        # print4xi(class2print, method2print, message2print)
        # -------------------------------------------------------------------------------------------------------------------------------------------
                
        message2print = "{(Step 1/3b) started" + "; repr-of-BoundedSemaphore4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
     
        if examine4xi(class2print, method2print):
        
            # -------------------------------------------------------------------------------------    
            # OK StepOriginal
            # -------------------------------------------------------------------------------------             
            message2print = "{(StepOriginal) calling" + "; BoundedSemaphore-repr-calling-by-init-of-BoundedSemaphore4Xi: " + str(BoundedSemaphore.__repr__) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       

            return BoundedSemaphore__repr__(self)
            
        else:    
        
            try:
                    
                # -------------------------------------------------------------------------------------------------------------------------------------------
                # message2print = "{(Step 2/3a) stepping" + "; (Step 2/3)-stepping-by-repr-of-BoundedSemaphore4Xi: " + "~~"  + "; time: " + time4xi() + "}"
                # print4xi(class2print, method2print, message2print)
                # -------------------------------------------------------------------------------------------------------------------------------------------
                
                 
                self._semlock = _multiprocessing.SemLock(get_semaphore_synchronize4xi(), get_sem_value_synchronize4xi(), get_sem_value_max_synchronize4xi())              
       
                value = self._semlock._get_value()
                
                message2print = "{(Step 2/3b) set" + "; value-set-by-repr-of-BoundedSemaphore4Xi to: " + str(value)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)         
            
            except Exception:
                # -------------------------------------------------------------------------------------    
                #
                # -------------------------------------------------------------------------------------                 
                message2print = "{(Step 2/3d) hit; unknown-value-hit-by-repr-of-BoundedSemaphore4Xi: " + "*** object has no attribute _semlock - to be solved !  ***" + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                
                message2print = "{(Step 2/3e) informing" + "; semlock-informing-by-repr-of-BoundedSemaphore4Xi: " + "_semlock is being used in the system as follows: " + \
                "\n" + "semlock4xi.py: sl = self._semlock = _multiprocessing.SemLock(kind2xi, value, maxvalue)" + \
                "\n" + "semlock4xi.py: self.acquire = self._semlock.acquire" + \
                "\n" + "semlock4xi.py: self.release = self._semlock.release" + \
                "\n" + "semlock4xi.py: self._semlock = _multiprocessing.SemLock._rebuild(*state)" + \
                "\n" + "condition4xi.py: num_waiters4xi = (self._sleeping_count._semlock._get_value()" + \
                "\n" + "condition4xi.py: assert self._lock._semlock._is_mine()" + \
                "\n" + "condition4xi.py: count2xi = self._lock._semlock._count()" + \
                "\n" + "joinablequeue4xi.py: if self._unfinished_tasks._semlock._is_zero()" + \
                "\n" + "lock4xi.py: if self._semlock._is_mine():" + \
                "\n" + "queue4xi.py: return self._sem._semlock._is_zero()" + \
                "\n" + "rlock4xi.py: if self._semlock._is_mine():" + \
                "\n" + "semaphore4xi.py: return self._semlock._get_value()  " + \
                "\n" + "; time: " + time4xi() + "}"
                
                print4xi(class2print, method2print, message2print)  
    
                value = 'unknown'
                
                raise
                # -------------------------------------------------------------------------------------    
                #
                # ------------------------------------------------------------------------------------- 
                                
                    
            # -------------------------------------------------------------------------------------------------------------------------------------------
            # message2print = "{(Step 3/3a) stepping" + "; (Step 3/3)-stepping-by-init-of-BoundedSemaphore4Xi: " + "~~"  + "; time: " + time4xi() + "}"
            # print4xi(class2print, method2print, message2print)
            # -------------------------------------------------------------------------------------------------------------------------------------------
                     
            message2print = "{(Step 3/3b) returning" + "; repr-returning-by-repr-of-BoundedSemaphore4Xi: see trace, please" + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)            
            # 
            
            return '<BoundedSemaphore4Xi(value=%s, maxvalue=%s)>' % \
                   (value, self._semlock.maxvalue)    

    #</methodize4xi>
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-BoundedSemaphore4Xi" usage = "senses BoundedSemaphore4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        senses BoundedSemaphore4Xi class objects
        '''
 
        
        class2print = 'BoundedSemaphore4Xi'
        method2print = 'sense'
        
        
        message2print = "{started" + "; sense-of-BoundedSemaphore4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
   

        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{sensing" + "; BoundedSemaphore4Xi-sensing-by-sense-of-BoundedSemaphore4Xi: " + str(BoundedSemaphore4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        kb2xi = sense_object4xi(self)

        try:
            kb2xi['inheritance-of-boundedsemaphore2xi'] = "Semaphore4Xi-inherited-by-boundedsemaphore2xi-of-BoundedSemaphore4Xi"
            kb2xi['value-of-boundedsemaphore2xi'] = self._semlock._get_value()
            kb2xi['maxvalue-of-boundedsemaphore2xi'] = self._semlock.maxvalue        
        except:
            message2print = "{yielded" + "; kb2xi-yielded-by-sense-of-BoundedSemaphore4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
        message2print = "{sensed" + "; BoundedSemaphore4Xi-sensed-by-sense-of-BoundedSemaphore4Xi: " + str(BoundedSemaphore4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb2xi-returning-by-sense-of-BoundedSemaphore4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)            
              
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-BoundedSemaphore4Xi" usage = "reasons BoundedSemaphore4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        reasons BoundedSemaphore4Xi class objects
        '''
     
        
        class2print = 'BoundedSemaphore4Xi'
        method2print = 'reason'

        
        message2print = "{started" + "; reason-of-BoundedSemaphore4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
         
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{reasoning" + "; BoundedSemaphore4Xi-reasoning-by-reason-of-BoundedSemaphore4Xi: "  + str(BoundedSemaphore4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        kb4xi = reason_object4xi(self)
        
        try:
            kb4xi['Exception-value-of-semlock-of-boundedsemaphore2xi'] = 'unknown'
              
        except:
            message2print = "{yielded" + "; kb4xi-yielded-by-reason-of-BoundedSemaphore4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
        
        message2print = "{reasoned" + "; BoundedSemaphore4Xi-reasoned-by-reason-of-BoundedSemaphore4Xi: "  + str(BoundedSemaphore4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb4xi-returning-by-reason-of-BoundedSemaphore4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)            
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-BoundedSemaphore4Xi" usage = "aces BoundedSemaphore4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        aces BoundedSemaphore4Xi class objects 
        '''
   
        
        
        class2print = 'BoundedSemaphore4Xi'
        method2print = 'ace'
        
        
        message2print = "{started" + "; ace-of-BoundedSemaphore4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
                
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{acing" + "; BoundedSemaphore4Xi-acing-by-ace-of-BoundedSemaphore4Xi: " + "BoundedSemaphore4Xi" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        
      
        kb8xi = ace_object4xi(self)  

        try:
            keys8xi = kb8xi(keys)
        
            found2xi = False
            for key8xi in keys8xi:
                if key8xi == 'Exception-value-of-semlock':
                    kb8xi['ace-of-boundedsemaphore2xi'] = 'ACK'
            
                    found2xi = True
                    
            if found2xi is False:
                kb8xi['ace-of-boundedsemaphore2xi'] = 'NACK'
        except:
            message2print = "{yielded" + "; kb8xi-yielded-by-ace-of-BoundedSemaphore4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                    
                
        
        message2print = "{aced" + "; BoundedSemaphore4Xi-aced-by-ace-of-BoundedSemaphore4Xi: " + "BoundedSemaphore4Xi" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb8xi-returning-by-ace-of-BoundedSemaphore4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)            
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------                            
    #</methodize4xi>
#</class4xi>
#</program4xi>     

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>


# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>

    
    

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of boundedsemaphore4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>

